namespace dkh {
	template <uint32_t D>
	TylerMapSource<D>::TylerMapSource (const char* parFilename) :
		base_class(m_stream->tellg()),
		m_stream(new std::ifstream(parFilename))
	{
		parse_map_data(*m_stream);
	}

	template <uint32_t D>
	TylerMapSource<D>::TylerMapSource (const std::string& parFilename) :
		base_class(m_stream->tellg()),
		m_stream(new std::ifstream(parFilename))
	{
		parse_map_data(*m_stream);
	}

	template <uint32_t D>
	TylerMapSource<D>::TylerMapSource (std::istream* parData) :
		base_class(m_stream->tellg()),
		m_stream(parData)
	{
		parse_map_data(*m_stream);
	}

	template <uint32_t D>
	void TylerMapSource<D>::parse_map_data (std::istream& parSrc) {
		std::vector<uint32_t> map_dims;
		std::vector<uint16_t> tile_dims;

		parseMapHeaders(parSrc, D, map_dims, tile_dims);
		DK_ASSERT(map_dims.size() == D);
		DK_ASSERT(tile_dims.size() == D);
		for (std::size_t z = 0; z < D; ++z) {
			m_map_size[z] = map_dims[z];
			m_tile_size[z] = tile_dims[z];
		}
	}

	template <uint32_t D>
	const typename TylerMapSource<D>::coords& TylerMapSource<D>::mapSize() const {
		return m_map_size;
	}

	template <uint32_t D>
	const typename TylerMapSource<D>::coords& TylerMapSource<D>::tileSize() const {
		return m_tile_size;
	}

	template <uint32_t D>
	void TylerMapSource<D>::fetch_raw (char* parOut, const coords& parFrom, const coords& parTo, std::size_t parSize) {
		(void)parOut;
		(void)parFrom;
		(void)parTo;
		(void)parSize;
	}

	template <uint32_t D>
	dk::MapTypes TylerMapSource<D>::mapType() const {
		return base_class::mapType();
	}

	template <uint32_t D>
	int TylerMapSource<D>::layersCount() const {
		return m_layer_count;
	}

	template <uint32_t D>
	void TylerMapSource<D>::chainedMaps (std::vector<std::string>& parOut) const {
		return base_class::chainedMaps(parOut);
	}

	template <uint32_t D>
	dk::HashType TylerMapSource<D>::layerTypeHash (int parIndex) const {
		return base_class::layerTypeHash(parIndex);
	}
} //namespace dkh
